@@ -1,5 +1,7 @@
 // -*- tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 // vi: set et ts=4 sw=2 sts=2:
+// SPDX-FileCopyrightInfo: Copyright (C) DUNE Project contributors, see file LICENSE.md in module root
+// SPDX-License-Identifier: LicenseRef-GPL-2.0-only-with-DUNE-exception
 #include "config.h"
 
 #include <algorithm>
@@ -10,8 +12,8 @@
 #include <dune/geometry/affinegeometry.hh>
 #include <dune/geometry/referenceelements.hh>
 
-#include <dune/localfunctions/lagrange/p1.hh>
-#include <dune/localfunctions/lagrange/q1.hh>
+#include <dune/localfunctions/lagrange/lagrangecube.hh>
+#include <dune/localfunctions/lagrange/lagrangesimplex.hh>
 
 #include <dune/localfunctions/dualmortarbasis/dualp1.hh>
 #include <dune/localfunctions/dualmortarbasis/dualq1.hh>
@@ -87,17 +89,16 @@ bool testBiorthogonality(const DualLfe& dualLfe, const LagrangeLfe& lagrangeLfe)
 template<class DualLfe,class LagrangeLfe>
 bool testFaceBiorthogonality(const DualLfe& dualLfe, const LagrangeLfe& lagrangeLfe)
 {
-  typedef typename DualLfe::Traits::LocalBasisType::Traits DualTraits;
-  typedef typename LagrangeLfe::Traits::LocalBasisType::Traits LagrangeTraits;
+  using DualTraits = typename DualLfe::Traits::LocalBasisType::Traits;
+  using LagrangeTraits = typename LagrangeLfe::Traits::LocalBasisType::Traits;
 
   const int dim = DualTraits::dimDomain;
   assert (dim == LagrangeTraits::dimDomain);
 
-  typedef typename DualTraits::DomainFieldType ctype;
-  typedef typename DualTraits::RangeFieldType field_type;
+  using ctype = typename DualTraits::DomainFieldType;
+  using field_type = typename DualTraits::RangeFieldType;
 
-  typedef Dune::ReferenceElement<ctype,dim> RefElement;
-  const RefElement& refElement = Dune::ReferenceElements<ctype,dim>::general(dualLfe.type());
+  const auto& refElement = Dune::ReferenceElements<ctype,dim>::general(dualLfe.type());
 
   const unsigned int numLagBasFct = lagrangeLfe.localBasis().size();
   const unsigned int numDualBasFct = dualLfe.localBasis().size();
@@ -118,9 +119,9 @@ bool testFaceBiorthogonality(const DualLfe& dualLfe, const LagrangeLfe& lagrange
   for (int i=0; i<refElement.size(1); i++) {
 
     // get geometry
-    const typename RefElement::template Codim<1>::Geometry& geometry = refElement.template geometry<1>(i);
+    const auto& geometry = refElement.template geometry<1>(i);
     // get a quadrature rule
-    const Dune::QuadratureRule<ctype,dim-1>& quad = Dune::QuadratureRules<ctype,dim-1>::rule(refElement.type(i,1),  dualLfe.localBasis().order() + lagrangeLfe.localBasis().order());
+    const auto& quad = Dune::QuadratureRules<ctype,dim-1>::rule(refElement.type(i,1),  dualLfe.localBasis().order() + lagrangeLfe.localBasis().order());
 
     // initialise with zeroes
     for (unsigned int k=0; k<numLagBasFct; k++)
@@ -130,14 +131,14 @@ bool testFaceBiorthogonality(const DualLfe& dualLfe, const LagrangeLfe& lagrange
 
     for(size_t j=0; j < quad.size(); j++) {
 
-      const Dune::FieldVector<ctype,dim-1>& pos = quad[j].position();
-      const Dune::FieldVector<ctype,dim>& elementPos = geometry.global(pos);
+      const auto& pos = quad[j].position();
+      const auto& elementPos = geometry.global(pos);
 
       // evaluate basis functions
       dualLfe.localBasis().evaluateFunction(elementPos,dualValues);
       lagrangeLfe.localBasis().evaluateFunction(elementPos,lagrangeValues);
 
-      ctype weight = quad[j].weight();
+      auto weight = quad[j].weight();
 
       for (unsigned int k=0; k<numLagBasFct; k++) {
 
@@ -149,9 +150,9 @@ bool testFaceBiorthogonality(const DualLfe& dualLfe, const LagrangeLfe& lagrange
     }
 
     // check if diagonal of mixedMassMat equals the lagrange integral and the off-diagonals are zero
-    for (unsigned int p=0; p<refElement.size(i,1,dim); p++) {
+    for (unsigned int p=0; p<(unsigned int)refElement.size(i,1,dim); p++) {
       int k= refElement.subEntity(i,1,p,dim);
-      for (unsigned int q=0; q<refElement.size(i,1,dim); q++) {
+      for (unsigned int q=0; q<(unsigned int)refElement.size(i,1,dim); q++) {
         int l= refElement.subEntity(i,1,q,dim);
         if(std::fabs(mixedMassMat[k][l] - (k==l)*(integralLagrange[k])) > TOL) {
           std::cout<<"for face "<<i<<" : \n";
@@ -166,31 +167,31 @@ bool testFaceBiorthogonality(const DualLfe& dualLfe, const LagrangeLfe& lagrange
   return biorthog;
 }
 
-int main(int argc, char** argv) try
+int main(int argc, char** argv)
 {
   bool success = true;
   Dune::DualP1LocalFiniteElement<double,double,1> dualP1lfem1D;
-  Dune::P1LocalFiniteElement<double,double,1> p1lfem1D;
+  Dune::LagrangeSimplexLocalFiniteElement<double,double,1,1> p1lfem1D;
   success = testBiorthogonality(dualP1lfem1D,p1lfem1D) and success;
 
   Dune::DualP1LocalFiniteElement<double,double,2> dualP1lfem2D;
-  Dune::P1LocalFiniteElement<double,double,2> p1lfem2D;
+  Dune::LagrangeSimplexLocalFiniteElement<double,double,2,1> p1lfem2D;
   success = testBiorthogonality(dualP1lfem2D,p1lfem2D) and success;
 
   Dune::DualP1LocalFiniteElement<double,double,3> dualP1lfem3D;
-  Dune::P1LocalFiniteElement<double,double,3> p1lfem3D;
+  Dune::LagrangeSimplexLocalFiniteElement<double,double,3,1> p1lfem3D;
   success = testBiorthogonality(dualP1lfem3D,p1lfem3D) and success;
 
   Dune::DualQ1LocalFiniteElement<double,double,1> dualQ1lfem1D;
-  Dune::Q1LocalFiniteElement<double,double,1> q1lfem1D;
+  Dune::LagrangeCubeLocalFiniteElement<double,double,1,1> q1lfem1D;
   success = testBiorthogonality(dualQ1lfem1D,q1lfem1D) and success;
 
   Dune::DualQ1LocalFiniteElement<double,double,2> dualQ1lfem2D;
-  Dune::Q1LocalFiniteElement<double,double,2> q1lfem2D;
+  Dune::LagrangeCubeLocalFiniteElement<double,double,2,1> q1lfem2D;
   success = testBiorthogonality(dualQ1lfem2D,q1lfem2D) and success;
 
   Dune::DualQ1LocalFiniteElement<double,double,3> dualQ1lfem3D;
-  Dune::Q1LocalFiniteElement<double,double,3> q1lfem3D;
+  Dune::LagrangeCubeLocalFiniteElement<double,double,3,1> q1lfem3D;
   success = testBiorthogonality(dualQ1lfem3D,q1lfem3D) and success;
 
   Dune::DualP1LocalFiniteElement<double,double,1,true> dualFaceP1lfem1D;
@@ -213,8 +214,3 @@ int main(int argc, char** argv) try
 
   return success ? 0 : 1;
 }
-catch(Dune::Exception e)
-{
-  std::cout<<e<<std::endl;
-  return 1;
-}
