@@ -1,5 +1,7 @@
 // -*- tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 // vi: set et ts=4 sw=2 sts=2:
+// SPDX-FileCopyrightInfo: Copyright (C) DUNE Project contributors, see file LICENSE.md in module root
+// SPDX-License-Identifier: LicenseRef-GPL-2.0-only-with-DUNE-exception
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -8,10 +10,11 @@
 #include <cstddef>
 #include <iostream>
 #include <ostream>
+#include <utility>
 
 #include <dune/common/exceptions.hh>
-#include <dune/common/forloop.hh>
 #include <dune/common/fvector.hh>
+#include <dune/common/hybridutilities.hh>
 
 #include <dune/geometry/type.hh>
 #include <dune/geometry/generalvertexorder.hh>
@@ -27,14 +30,12 @@ static const double eps = 1e-9;
 static const double delta = 1e-5;
 
 template<int k>
-struct Test {
-
-  static void apply(int &result) {
+static void test(int &result)
+{
     std::cout << "== Checking global-valued Pk2D elements (with k=" << k << ")"
               << std::endl;
 
-    Dune::GeometryType gt;
-    gt.makeTriangle();
+    Dune::GeometryType gt(Dune::GeometryTypes::triangle);
 
     typedef TestGeometries<double, 2> TestGeos;
     static const TestGeos testGeos;
@@ -53,15 +54,14 @@ struct Test {
       result = 0;
     else
       result = 1;
-  }
-};
+}
 
 int main(int argc, char** argv) {
   try {
     int result = 77;
 
-    static const std::size_t max_k = 20;
-    Dune::ForLoop<Test, 0, max_k>::apply(result);
+    static constexpr std::size_t max_k = 20;
+    Dune::Hybrid::forEach(std::make_index_sequence<max_k+1>{},[&](auto i){test<i>(result);});
 
     return result;
   }
