@@ -1,5 +1,7 @@
-// -*- tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 2 -*-
-// vi: set et ts=8 sw=2 sts=2:
+// -*- tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*-
+// vi: set et ts=4 sw=2 sts=2:
+// SPDX-FileCopyrightInfo: Copyright (C) DUNE Project contributors, see file LICENSE.md in module root
+// SPDX-License-Identifier: LicenseRef-GPL-2.0-only-with-DUNE-exception
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -13,7 +15,6 @@
 #include <dune/common/fvector.hh>
 
 #include <dune/geometry/type.hh>
-#include <dune/geometry/mockgeometry.hh>
 
 #include <dune/localfunctions/lagrange/q1.hh>
 
@@ -30,8 +31,7 @@ void testQ1(int &result) {
   std::cout << "== Checking global-valued Q1 elements (with dim=" << dim << ")"
             << std::endl;
 
-  Dune::GeometryType gt;
-  gt.makeCube(dim);
+  Dune::GeometryType gt(Dune::GeometryTypes::cube(dim));
 
   typedef TestGeometries<double, dim> TestGeos;
   static const TestGeos testGeos;
