@@ -1,20 +1,23 @@
 // -*- tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 // vi: set et ts=4 sw=2 sts=2:
+// SPDX-FileCopyrightInfo: Copyright (C) DUNE Project contributors, see file LICENSE.md in module root
+// SPDX-License-Identifier: LicenseRef-GPL-2.0-only-with-DUNE-exception
 #include <config.h>
 
 #undef DUNE_VIRTUAL_SHAPEFUNCTIONS
 
-
+#include <array>
 #include <cstddef>
 #include <iostream>
 
 #include <dune/geometry/type.hh>
 #include <dune/localfunctions/common/virtualinterface.hh>
+#include <dune/localfunctions/common/virtualwrappers.hh>
 
 #include <dune/localfunctions/lagrange/p0.hh>
-#include <dune/localfunctions/lagrange/p1.hh>
+#include <dune/localfunctions/lagrange/lagrangesimplex.hh>
 #include <dune/localfunctions/lagrange/pq22d.hh>
-#include <dune/localfunctions/monom.hh>
+#include <dune/localfunctions/monomial.hh>
 
 /** \file
     \brief Test the dynamically polymorphic shape function interface
@@ -31,11 +34,17 @@ void syntax_check( const T& )
 {}
 
 // A test function to test the local interpolation
-template <class DomainType, class RangeType>
+template <class D, class R>
 struct TestFunction
-//    : public VirtualFunction<DomainType,RangeType>
-  : public Function<const DomainType&,RangeType&>
 {
+  using DomainType = D;
+  using RangeType = R;
+
+  struct Traits {
+    using DomainType = D;
+    using RangeType = R;
+  };
+
   void evaluate(const DomainType& in, RangeType& out) const {
     // May not be flexible enough to compile for all range types
     out = 1;
@@ -48,7 +57,7 @@ void testLocalBasis(const LocalBasisVirtualInterface<T>* localBasis)
 {
   // call each method once to test that it's there
   syntax_check<unsigned int>( localBasis->order() );
-  unsigned int size DUNE_UNUSED = localBasis->size();
+  [[maybe_unused]] unsigned int size = localBasis->size();
 
   // evaluate the local basis at (0,...,0)
   typename T::DomainType in(0);
@@ -86,45 +95,26 @@ void testLocalInterpolation(const LocalInterpolationVirtualInterface<DomainType,
   // Test interpolation of a function object derived from VirtualFunction
   TestFunction<DomainType,RangeType> testFunction;
   std::vector<typename RangeType::field_type> coefficients;
+
+  //////////////////////////////////////////////////////////////////////////////
+  //  Part A: Feed the function to the 'interpolate' method in form of
+  //    a class providing an evaluate() method.
+  //    This way is deprecated since dune-localfunctions 2.7.
+  //////////////////////////////////////////////////////////////////////////////
   localInterpolation->interpolate(testFunction, coefficients);
+
+  //////////////////////////////////////////////////////////////////////////////
+  //  Part B: Redo the same test, but feed the function to the
+  //    'interpolate' method in form of a callable.
+  //////////////////////////////////////////////////////////////////////////////
+  auto callableTestFunction = [&](const auto& x) {
+    RangeType y(0);
+    testFunction.evaluate(x,y);
+    return y;
+  };
+  localInterpolation->interpolate(callableTestFunction, coefficients);
 }
 
-template <class Interface, int order>
-struct EvaluateTest
-{
-  static void test(const Interface& fe)
-  {
-    typedef typename Interface::Traits::LocalBasisType::Traits LBTraits;
-
-    Dune::array<int,order> d;
-    for(unsigned int i=0; i<d.size(); ++i)
-      d[i] = 0;
-
-    typename LBTraits::DomainType x;
-    x = 0;
-
-    typename std::vector<typename LBTraits::RangeType> y1;
-    typename std::vector<typename LBTraits::RangeType> y2;
-
-    fe.localBasis().evaluate(d,x,y1);
-    fe.localBasis().template evaluate<order>(d,x,y2);
-
-    for(unsigned int i=0; i<d.size(); ++i)
-      if (y1[i] != y2[i])
-        DUNE_THROW(Dune::Exception, "result of template evaluate<order>() and virtual evaluate() do not coincide");
-
-    EvaluateTest<Interface, order-1>::test(fe);
-  }
-};
-
-template <class Interface>
-struct EvaluateTest<Interface, -1>
-{
-  static void test(const Interface& fe)
-  {}
-};
-
-
 // Test all methods of a local finite element given as a pointer to the abstract base class
 template <class T>
 void testLocalFiniteElement(const LocalFiniteElementVirtualInterface<T>* localFiniteElement)
@@ -146,52 +136,50 @@ void testLocalFiniteElement(const LocalFiniteElementVirtualInterface<T>* localFi
   const typename FEType::Traits::LocalInterpolationType& interp = localFiniteElement->localInterpolation();
   testLocalInterpolation(&interp);
 
-  EvaluateTest<FEType, T::diffOrder>::test(*localFiniteElement);
+  // Test cloning
+  const LocalFiniteElementVirtualInterface<T>* other = localFiniteElement->clone();
+
+  // Make sure new object has the same type
+  if (typeid(other).hash_code() != typeid(localFiniteElement).hash_code())
+    DUNE_THROW(Dune::Exception, "'clone' method returns object of wrong type");
+
+  // But it shouldn't be the same object
+  if (other==localFiniteElement)
+    DUNE_THROW(Dune::Exception, "'clone' method returned the object it was called for!");
+
+  delete other;
+
 }
 
-int main (int argc, char *argv[]) try
+int main (int argc, char *argv[])
 {
 
-  typedef Dune::P1LocalFiniteElement<double, double, 2>::Traits::LocalBasisType::Traits LBTraits;
-  typedef Dune::FixedOrderLocalBasisTraits<LBTraits,0>::Traits C0LBTraits;
-  typedef Dune::FixedOrderLocalBasisTraits<LBTraits,1>::Traits C1LBTraits;
-  typedef Dune::FixedOrderLocalBasisTraits<LBTraits,2>::Traits C2LBTraits;
+  typedef Dune::LagrangeSimplexLocalFiniteElement<double, double, 2, 1>::Traits::LocalBasisType::Traits LBTraits;
 
-  const Dune::P0LocalFiniteElement<double, double, 2> p0FE(Dune::GeometryType(Dune::GeometryType::cube, 2));
+  const Dune::P0LocalFiniteElement<double, double, 2> p0FE(Dune::GeometryTypes::cube(2));
   const Dune::LocalFiniteElementVirtualImp<Dune::P0LocalFiniteElement<double, double, 2> > p0VFE(p0FE);
-  testLocalFiniteElement<C0LBTraits>(&p0VFE);
+  testLocalFiniteElement<LBTraits>(&p0VFE);
 
-  const Dune::PQ22DLocalFiniteElement<double, double> pq2FE(Dune::GeometryType(Dune::GeometryType::cube, 2));
+  const Dune::PQ22DLocalFiniteElement<double, double> pq2FE(Dune::GeometryTypes::cube(2));
   const Dune::PQ22DLocalFiniteElement<double, double> pq2FE2(pq2FE);
 
   const Dune::LocalFiniteElementVirtualImp<Dune::PQ22DLocalFiniteElement<double, double> > pq2VFE(pq2FE);
-  testLocalFiniteElement<C0LBTraits>(&pq2VFE);
+  testLocalFiniteElement<LBTraits>(&pq2VFE);
 
-  const Dune::LocalFiniteElementVirtualImp<Dune::P1LocalFiniteElement<double, double, 2> > p1VFE;
-  testLocalFiniteElement<C0LBTraits>(&p1VFE);
-  testLocalFiniteElement<C1LBTraits>(&p1VFE);
-  testLocalFiniteElement<C2LBTraits>(&p1VFE);
+  const Dune::LocalFiniteElementVirtualImp<Dune::LagrangeSimplexLocalFiniteElement<double, double, 2, 1>> p1VFE;
+  testLocalFiniteElement<LBTraits>(&p1VFE);
 
-  const Dune::LocalFiniteElementVirtualImp<
-      Dune::LocalFiniteElementVirtualImp<Dune::P1LocalFiniteElement<double, double, 2> > > p1VVFE;
-  testLocalFiniteElement<C0LBTraits>(&p1VVFE);
-  testLocalFiniteElement<C1LBTraits>(&p1VVFE);
-  testLocalFiniteElement<C2LBTraits>(&p1VVFE);
+  const Dune::LocalFiniteElementVirtualImp<Dune::LocalFiniteElementVirtualImp<
+    Dune::LagrangeSimplexLocalFiniteElement<double, double, 2, 1>>> p1VVFE;
+  testLocalFiniteElement<LBTraits>(&p1VVFE);
 
-  typedef Dune::MonomLocalFiniteElement<double, double, 2, 7> Monom7;
-  const Monom7 monom7FE(Dune::GeometryType(Dune::GeometryType::cube, 2));
+  typedef Dune::MonomialLocalFiniteElement<double, double, 2, 7> Monom7;
+  const Monom7 monom7FE(Dune::GeometryTypes::cube(2));
   const Dune::LocalFiniteElementVirtualImp<Monom7> monom7VFE(monom7FE);
   const Dune::LocalFiniteElementVirtualImp<
       Dune::LocalFiniteElementVirtualImp<Monom7> > monom7VVFE(monom7VFE);
-  testLocalFiniteElement<C0LBTraits>(&monom7VVFE);
-  testLocalFiniteElement<C1LBTraits>(&monom7VVFE);
-  testLocalFiniteElement<C2LBTraits>(&monom7VVFE);
+  testLocalFiniteElement<LBTraits>(&monom7VVFE);
 
   return 0;
 
 }
-catch (Exception e) {
-
-  std::cout << e << std::endl;
-  return 1;
-}
